package app.window;

import app.SettingApp;
import app.engine.Engine;
import app.engine.Game;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;

/**
 * @author Death
 */
public class MainPanel extends JPanel{

    Game game;
    public Engine background;
    
    private Mouse mouse;
    private Keyboard keyboard;
    private FPS fps;
    
    private final int FPS_F = 60;
    private final long DELAY_L = 1000 / FPS_F;
    
    public MainPanel(){
        super();
        this.setPreferredSize(new Dimension(SettingApp.WIDTH, SettingApp.HEIGHT));
        this.setFocusable(true);
        mouse = new Mouse(this);
        keyboard = new Keyboard(this);
        fps = new FPS();
        
        this.addKeyListener(keyboard);
        this.addMouseListener(mouse);
        this.addMouseMotionListener(mouse);
        this.addMouseWheelListener(mouse);
        
        background = null;
    }
    
    public void addGame(Game game){
        game.init();
        this.game = game;
        startTime = System.currentTimeMillis();
        lag = 0;
    }

    private long startTime, lag, currentTime, elapsed;
    
    @Override
    protected void paintComponent(Graphics g){
        super.paintComponent(g); //To change body of generated methods, choose Tools | Templates.
        Graphics2D g2 = (Graphics2D) g;
        
        currentTime = System.currentTimeMillis();
        elapsed = currentTime - startTime;
        startTime = currentTime;
        lag += elapsed;
        
        if(SettingApp.ANTIALIASING) g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        g2.setColor(new Color(100, 100, 100));
        g2.fillRect(0, 0, SettingApp.WIDTH, SettingApp.HEIGHT);
        
        while(lag >= DELAY_L){
            if(background != null)
                background.update();
            if(game != null)
                game.update();
            lag -= DELAY_L;
        }
        
        
        if(background != null)
            background.paint(g2);
        if(game != null)
            game.paint(g2);
        
        
        if(SettingApp.VISIBLE_FPS){
            fps.update();
            fps.paint(g2);
        }
        
        super.repaint();
        sleep();
    }
    
    private void sleep(){
        try{
            Thread.currentThread().sleep(1);
        }
        catch(InterruptedException ex){
            Logger.getLogger(MainPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public void doLayout(){
        super.doLayout(); //To change body of generated methods, choose Tools | Templates.
        SettingApp.WIDTH = this.getWidth();
        SettingApp.HEIGHT = this.getHeight();
    }
}
