package ws.player;

import app.SettingApp;
import app.util.SM;
import app.window.Keyboard;
import app.window.Mouse;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import ws.gameField.GameField;
import ws.gameOver.GameOverScreen;
import ws.menu.MenuScreen;
import ws.obj.Body;

/**
 * @author Death
 */
public class Player extends Body{
    
    public static int UP = 0, DOWN = 1, RIGHT = 2, LEFT = 3;
    public static int REDC = 0, GREENC = 255, BLUEC = 0;
    
    GameField gameField;
    
    boolean keys[] = new boolean[4];
    
    public int damageBullet;
    public int speedBullet;
    public int rBullet;
    public ArrayList<Bullet> bullets;
    boolean isFire;
    
    long timeDelayFire, timeDelayMoving;

    public Player(GameField gameField){
        super();
        this.gameField = gameField;
        
        speed = 7;
        healthMax = 100;
        health = healthMax;
        r = 10;
        x = SettingApp.WIDTH / 2 - r;
        y = SettingApp.HEIGHT - r * 4;
        damage = 1;
        color = new Color(REDC, GREENC, BLUEC);
        
        damageBullet = 1;
        speedBullet = 10;
        rBullet = 1;
        bullets = new ArrayList<>();
        isFire = false;
        invulnerable = false;
        timeExpectationInvulnerable = 100;
    }

    @Override
    public void paint(Graphics2D g){
        super.paint(g); //To change body of generated methods, choose Tools | Templates.
        drawHealth(g);
        paintBullet(g);
    }
    
    private void paintBullet(Graphics2D g){
        for(int i = 0; i < bullets.size(); i++){
            bullets.get(i).paint(g);
        }
    }
    
    @Override
    public boolean removeHealth(int a){
        boolean s = super.removeHealth(a);
        if(s) gameField.gameScreen.game.setScreen(new GameOverScreen());
        return false;
    }
    
    
    @Override
    public void update(){
        super.update();
        updateInvulnerable();
        updateBullet();
        moving();
        color = SM.set(REDC, GREENC, BLUEC, color.getAlpha());
    }
    
    public void deathEnemy(int i){
        if(i % 5 == 0){
            health += 1;
            if(health > healthMax) health = healthMax;
        }
        if(i % 10 == 0){
            damageBullet += 1;
            speedBullet += 1;
            if(speedBullet > 100) speedBullet = 100;
            if(damageBullet > 50) damageBullet = 50;
        }
        
        if(i % 20 == 0){
            r ++;
            healthMax += 5;
            health += 5;
            if(r > 30) r = 30;
            if(healthMax > 300) healthMax = 300;
            if(health > 300) health = 300;
        }
        
        if(i % 50 == 0){
            speed += 1;
            if(speed > 12) speed = 12;
        }
        
        if(i > 400) rBullet = 2;
    }
    
    private void updateBullet(){
        for(int i = 0; i < bullets.size(); i++){
            bullets.get(i).update();
        }
        
        if(isFire){
            if(System.currentTimeMillis() - timeDelayFire > 100){
                timeDelayFire = System.currentTimeMillis();
                if(gameField.enemyPool.deathEnemy > 250){
                    Bullet b1 = new Bullet(this);
                    b1.x -= r;
                    Bullet b2 = new Bullet(this);
                    b2.x += r;
                    Bullet b3 = new Bullet(this);
                    b3.y -= r;
                    
                    bullets.add(b1);
                    bullets.add(b2);
                    bullets.add(b3);
                }else if(gameField.enemyPool.deathEnemy > 100){
                    Bullet b1 = new Bullet(this);
                    b1.x -= r / 2;
                    Bullet b2 = new Bullet(this);
                    b2.x += r / 2;
                    
                    bullets.add(b1);
                    bullets.add(b2);
                }else{
                    bullets.add(new Bullet(this));
                }
            }
        }
    }
    
    private void moving(){
        if(!(System.currentTimeMillis() - timeDelayMoving > 1000/50))
            return;
        timeDelayMoving = System.currentTimeMillis();
        if(keys[LEFT]){
            this.x -= speed;
        }
        if(keys[RIGHT]){
            this.x += speed;
        }
        if(keys[UP]){
            this.y -= speed;
        }
        if(keys[DOWN]){
            this.y += speed;
        }
        if(x - r < 0) x = 0 + r;
        if(x + r > SettingApp.WIDTH) x = SettingApp.WIDTH - r;
        if(y + r > SettingApp.HEIGHT) y = SettingApp.HEIGHT - r;
    }

    public void keyLst(int key, int type){
        if(key == KeyEvent.VK_W && type == Keyboard.PRESSED){
            keys[UP] = true;
        }
        if(key == KeyEvent.VK_S && type == Keyboard.PRESSED){
            keys[DOWN] = true;
        }
        if(key == KeyEvent.VK_A && type == Keyboard.PRESSED){
            keys[LEFT] = true;
        }
        if(key == KeyEvent.VK_D && type == Keyboard.PRESSED){
            keys[RIGHT] = true;
        }
        
//        if(key == KeyEvent.VK_E && type == Keyboard.PRESSED){
//            gameField.deathEnemy(++gameField.enemyPool.deathEnemy);
//        }
//        
//        
        if(key == KeyEvent.VK_ESCAPE && type == Keyboard.PRESSED){
            this.gameField.gameScreen.game.setScreen(new MenuScreen());
        }
        
        if(key == KeyEvent.VK_W && type == Keyboard.RELEASED){
            keys[UP] = false;
        }
        if(key == KeyEvent.VK_S && type == Keyboard.RELEASED){
            keys[DOWN] = false;
        }
        if(key == KeyEvent.VK_A && type == Keyboard.RELEASED){
            keys[LEFT] = false;
        }
        if(key == KeyEvent.VK_D && type == Keyboard.RELEASED){
            keys[RIGHT] = false;
        }
    }

    public void mouseLst(int key, int type){
        if(key == MouseEvent.BUTTON1 && type == Mouse.PRESSED){
            isFire = true;
        }
        if(key == MouseEvent.BUTTON1 && type == Mouse.RELEASED){
            isFire = false;
        }
    }
}
